from datetime import datetime

from pydantic import BaseModel, Field
from quart import Blueprint
from quart_schema import validate_response

from foxnouns.settings import BASE_DOMAIN
from foxnouns.models.user import SelfUserModel


bp = Blueprint("auth_v2", __name__)


class URLsResponse(BaseModel):
    discord: str | None = Field(default=None)
    google: str | None = Field(default=None)
    tumblr: str | None = Field(default=None)


@bp.post("/api/v2/auth/urls", host=BASE_DOMAIN)
@validate_response(URLsResponse, 200)
async def urls():
    # TODO: build authorization URLs + callback URLs, store state in Redis
    raise NotImplementedError()


class OAuthCallbackRequest(BaseModel):
    callback_domain: str
    code: str
    state: str


class BaseCallbackResponse(BaseModel):
    """The base class for callback responses."""

    has_account: bool


class ExistingUserCallbackResponse(BaseCallbackResponse):
    """The class returned when a user already exists."""

    token: str
    user: SelfUserModel


class NewUserCallbackResponse(BaseCallbackResponse):
    """The class returned when the user is new and has to create an account."""

    remote_username: str
    ticket: str
    require_captcha: bool


class DeletedUserCallbackResponse(BaseCallbackResponse):
    """The class returned when the user has been deleted."""

    token: str
    user: SelfUserModel

    deleted_at: datetime
    self_delete: bool
    delete_reason: str | None = Field(default=None)
