from pydantic import BaseModel
from quart import Blueprint
from quart_schema import validate_response
from sqlalchemy import select
from sqlalchemy.sql import func

from foxnouns.db import Member, User
from foxnouns.db.aio import async_session
from foxnouns.settings import BASE_DOMAIN

bp = Blueprint("meta_v2", __name__)


class MetaResponse(BaseModel):
    users: int
    members: int


@bp.get("/api/v2/meta", host=BASE_DOMAIN)
@validate_response(MetaResponse)
async def meta():
    async with async_session() as session:
        user_count = await session.scalar(select(func.count()).select_from(User))
        member_count = await session.scalar(select(func.count()).select_from(Member))

        return MetaResponse(users=user_count, members=member_count)
